@@ -185,6 +185,7 @@ bamExperiment(bv)
 ###################################################
 ### code chunk number 22: readGAlignmentsFromBam (eval = FALSE)
 ###################################################
+## library(GenomicAlignments)
 ## library(parallel)
 ## options(srapply_fapply="parallel", mc.cores=detectCores())
 ## olaps <- readGAlignmentsFromBam(bv)
@@ -193,6 +194,7 @@ bamExperiment(bv)
 ###################################################
 ### code chunk number 23: olaps
 ###################################################
+library(GenomicAlignments)
 load(system.file("extdata", "olaps.Rda", package="Rsamtools"))
 olaps
 head(olaps[[1]])
