@@ -4,8 +4,7 @@
 #include <Rinternals.h>
 
 SEXP read_bam_header(SEXP fname, SEXP mode);
-SEXP scan_bam_template(SEXP tags);
-SEXP as_bam(SEXP fname, SEXP destination);
+SEXP scan_bam_template(SEXP rname, SEXP tags);
 SEXP sort_bam(SEXP fname, SEXP destinationPrefix, SEXP isByQname,
               SEXP maxMemory);
 SEXP merge_bam(SEXP fnames, SEXP destination, SEXP overwrite,
